package org.jingle.util.dydelegation;

import java.lang.reflect.Method;

/**
 * The Delegation Invocation Handler
 *
 * All the invocation to the generated delegation object will redirect to this handler
 * @author JianLu
 * @version 1.0 2004-9-23
 * @since 1.0
 */
public interface DelegationInvocationHandler {
    /**
     * This method will me called before the the invocation on the delegated object
     * @param bean The delegated object
     * @param method The delagtion method call
     * @param args The method call arguments
     * @return true or false. If true, the invocation on the object to be delegated will be executed. Otherwise, it will be skipped.
     * @throws Throwable Any possible Exception/Error. If any Exception/Error throwed, the delegation method call will return immediately.
     */
    public boolean invokeBefore(Object bean, Method method, Object[] args) throws Throwable;

    /**
     * This method will be called if the invocation on the delegated object returns without any Exception/Error. If the invocation on delegated object is skipped, it will be called after invokeBefore().
     * @param bean The delegated object
     * @param method The deleagtion method call
     * @param args The method call arguments
     * @param result The return object of the invocation of delegated object 
     * @return The return object of the delegation method
     * @throws Throwable Any possible Exception/Error. It will be treated as the Exception/Error of the delegation method.
     */
    public Object invokeAfter(Object bean, Method method, Object[] args, Object result) throws Throwable;

    /**
     * This method will be called when the invocation on the delegated object throws Exception/Error. If the invocation on delegated object is skipped, it will be skipped also.
     * @param bean The delegated object
     * @param method The deleagtion method call
     * @param args The method call arguments
     * @param throwable The Throwable object throwed by the invocation of delegated object.
     * @return The return object of the delegation method
     * @throws Throwable Any possible Exception/Error. It will be treated as the Exception/Error of the delegation method.
     */
    public Object invokeAfterException(Object bean, Method method, Object[] args, Throwable throwable) throws Throwable;
}
